package pl.edu.icm.yadda.service2.user.domain;

import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.io.Resource;


/**
 * XSLT based domain overrider.
 * @author mhorst
 *
 */
public class XSLTDomainOverrider implements DomainOverrider, InitializingBean {
	
	/**
	 * XSL resource.
	 */
	protected Resource xslResource;


	private Transformer transformer;
	
	/* (non-Javadoc)
	 * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
	 */
	@Override
	public void afterPropertiesSet() throws Exception {
		TransformerFactory tfactory = TransformerFactory.newInstance();
		InputStream inputStream = xslResource.getInputStream();
		try {
			transformer = tfactory.newTransformer(new StreamSource(inputStream));	
		} finally {
			if (inputStream!=null) {
				inputStream.close();
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see pl.edu.icm.yadda.service2.user.DomainOverrider#overrideDomain(java.lang.String, java.lang.String)
	 */
	@Override
	public String overrideDomain(String data, String domain) {
		try {
			transformer.setParameter("targetDomain", domain);
			Source xmlSource = new StreamSource(new StringReader(data));
			StringWriter w = new StringWriter();
			transformer.transform(xmlSource, new StreamResult(w));
			return w.toString();
		} catch (TransformerException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Sets xsl resource.
	 * @param xslResource
	 */
	public void setXslResource(Resource xslResource) {
		this.xslResource = xslResource;
	}

}
