package pl.edu.icm.yadda.service2.user.log;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Stateful CSV migration logger.
 * @author mhorst
 *
 */
public class CSVMigrationLogger implements MigrationLogger {
	
	/**
	 * Root directory.
	 */
	protected String rootDir;
	
	private String currentWorkFile;
	
	private List<String> entries = new ArrayList<String>();

	
	/* (non-Javadoc)
	 * @see pl.edu.icm.yadda.service2.user.log.MigrationLogger#init()
	 */
	@Override
	public void init() {
		try {
//			creating directory
			currentWorkFile = rootDir;
			if (!rootDir.endsWith(File.separator)) {
				currentWorkFile += File.separator;
			}
			currentWorkFile +="user-data-migration-logs";
			currentWorkFile += File.separator;
			new File(currentWorkFile).mkdirs();
			currentWorkFile += System.currentTimeMillis() + ".csv";
			new File(currentWorkFile).createNewFile();
		} catch (IOException e) {
			throw new RuntimeException(
					"unable to create file: " + currentWorkFile, e);
		}
	}
	
	/* (non-Javadoc)
	 * @see pl.edu.icm.yadda.service2.user.log.MigrationLogger#log(pl.edu.icm.yadda.service2.user.log.MigrationLogger.Type, pl.edu.icm.yadda.service2.user.log.MigrationLogger.Status, java.lang.String, java.lang.String, java.lang.String[])
	 */
	@Override
	public void log(Type type, Status status, String id, String msg,
			String... auxParams) {
		StringBuffer strBuff = new StringBuffer();
		strBuff.append(type);
		strBuff.append(';');
		strBuff.append(status);
		strBuff.append(';');
		strBuff.append(id);
		strBuff.append(';');
		strBuff.append(msg);
		if (auxParams!=null) {
			strBuff.append(';');
			boolean isFirst = true;
			for (String param : auxParams) {
				if (isFirst) {
					isFirst = false;
				} else {
					strBuff.append(';');
				}
				strBuff.append(param);
			}
		}
		entries.add(strBuff.toString());
	}

	/* (non-Javadoc)
	 * @see pl.edu.icm.yadda.service2.user.log.MigrationLogger#close()
	 */
	@Override
	public void close() {
		if (!entries.isEmpty()) {
			try {
				FileWriter writer = new FileWriter(new File(currentWorkFile));
				try {
					for (String entry : entries) {
						writer.append(entry + '\n');
					}	
				} finally {
					writer.close();
				}
			} catch (IOException e) {
				throw new RuntimeException("unable to write to a file: " + 
						currentWorkFile, e);
			}
		}
	}

	/**
	 * Sets root directory.
	 * @param rootDir
	 */
	public void setRootDir(String rootDir) {
		this.rootDir = rootDir;
	}

}
